//////////////////////////////////////////////////////////////////////////// 
// Signal_Processing_Supplementary_toolbox
//
// Copyright (c) by Y. K. Singh, CDAC, Kolkata.  2012
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [ n , wn]=buttord(wp , ws , rp, rs, opt)
    //wp normalized passband edge frequency in radian per sample
    //ws normalized stopband edge frequency in radian per sample 
    //rp passband ripple in dB
    //rs stopband attenuation in dB
    //opt may be 's' for analog and 'z' for digital 
    //by default opt='z'
    [out,inp]=argn(0);
    if inp==4 then
        opt='z';
    end
    ftype=2*(length(wp)-1);
    if wp(1)<ws(1) then
        ftype=ftype+1;
    else
        ftype=ftype+2;
    end

    if ~strcmp(opt,'z') then
        WP=tan(%pi*wp/2);
        WS=tan(%pi*ws/2);
    else
        WP=wp;
        WS=ws;
    end
    
    if ftype==1 then
        WA=WS/WP;
    elseif ftype==2 then
        WA=WP/WS;
    elseif ftype==3 then
        fo = optimset();
        fo.Display='none';
	wp1 = lclfminbnd('bscost',WP(1),WS(1)-1e-12,fo,1,WP,WS,rs,rp,'butter');
	WP(1) = wp1;
	wp2 = lclfminbnd('bscost',WS(2)+1e-12,WP(2),fo,2,WP,WS,rs,rp,'butter');
	WP(2) = wp2;
	WA=(WS*(WP(1)-WP(2)))./(WS.^2 - WP(1)*WP(2));
    else
        WA=(WS.^2-WP(1)*WP(2))./(WS*(WP(1)-WP(2)));
    end
    WA=min(abs(WA));
    n=ceil( log10( (10 .^ (0.1*abs(rs)) - 1)./ ...
	(10 .^ (0.1*abs(rp)) - 1) ) / (2*log10(WA)) );
    W0 = WA / ( (10^(.1*abs(rs)) - 1)^(1/(2*(abs(n)))));
    if ftype==1 then
        WN=WP*W0;
    elseif ftype ==2 then
        WN=WP/W0;
    elseif ftype==3 then
        WN(1) = ( (WP(2)-WP(1)) + sqrt((WP(2)-WP(1))^2 + 4*W0.^2*WP(1)*WP(2)))./(2*W0);
        WN(2) = ( (WP(2)-WP(1)) - sqrt((WP(2)-WP(1))^2 + 4*W0.^2*WP(1)*WP(2)))./(2*W0);
        WN=gsort(abs(WN)); 
    else
        W0=[-W0 W0];
        WN=-W0*(WP(2)-WP(1))/2 + sqrt(W0.^2/4*(WP(2)-WP(1))^2+WP(1)*WP(2));
        WN=gsort(abs(WN));
    end
if ~strcmp(opt,'z') then
     wn=(2/%pi)*atan(WN);
else
     wn=WN(:)';
end
endfunction