//////////////////////////////////////////////////////////////////////////// 
// Signal_Processing_Supplementary_toolbox
//
// Copyright (c) by Y. K. Singh, CDAC, Kolkata.  2012
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [order,wn] = cheb2ord(wp,ws,rp,rs,opt)

[nargout,nargin]=argn(0);
if nargin == 4 then
   opt = 'z';
elseif nargin == 5 then
 if ~strcmp(opt,'z') & ~strcmp(opt,'s') then 
    error('Invalid option for final argument.');
   end
end

msg=freqchk(wp,ws,opt);
if strcmp(msg,'') then
    error(msg);
end
// figure out filter type
ftype = 2*(length(wp) - 1);
if wp(1) < ws(1) then
    ftype = ftype + 1;    // low (1) or reject (3)
else
    ftype = ftype + 2;    // high (2) or pass (4)
end

// first, prewarp frequencies from digital (unit circle) to analog (imag. axis):
if ~strcmp(opt,'z')  then   // digital
    WPA=tan(%pi*wp/2);
    WSA=tan(%pi*ws/2);
else  // don't have to if analog already
    WPA=wp;
    WSA=ws;
end

// next, transform to low pass prototype with passband edge of 1 and stopband
// edges determined by the following: (see Rabiner and Gold, p.258)
if ftype == 1    then // low
    WA=WSA/WPA;
elseif ftype == 2  then  // high
    WA=WPA/WSA;
elseif ftype == 3  then //   stop
    fo=optimset();
    fo.Display='none';
    wp1 = lclfminbnd('bscost',WPA(1),WSA(1)-1e-12,fo,1,..
                   WPA,WSA,rs,rp,'cheby');
    WPA(1) = wp1;
    wp2 = lclfminbnd('bscost',WSA(2)+1e-12,WPA(2),fo,2,..
                   WPA,WSA,rs,rp,'cheby');
    WPA(2) = wp2;
    WA=(WSA*(WPA(1)-WPA(2)))./(WSA.^2 - WPA(1)*WPA(2));
elseif ftype == 4   then //  pass
    WA=(WSA.^2 - WPA(1)*WPA(2))./(WSA*(WPA(1)-WPA(2)));
end

//find the minimum order cheby. type 2 filter to meet the more demanding spec:
WA=min(abs(WA));
order=ceil(acosh(sqrt((10^(.1*abs(rs))-1)/(10^(.1*abs(rp))-1)))/acosh(WA));
//ref: M.E. Van Valkenburg, "Analog Filter Design", p.232, eqn 8.39

// next find the frequency "new_wp" at which the response of an analog low-pass
// prototype is exactly -rp dB.  The prototype is the one for which the beginning
// of the stop-band is at frequency 1.
// (new_wp will be less than 1):
new_wp=1/cosh(1/order*acosh(sqrt((10^(.1*abs(rs)) - 1)/(10^(.1*abs(rp)) - 1))));

// Now convert the stop band frequency back from lowpass prototype 
// to the original analog filter.
// Here we use the mapping which maps the frequency "new_wp" to the original WP, 
// to map the (+/-)1 frequency to WN (WN will be between WP and WS):
    if ftype == 1  then  // low
    WN=WPA/new_wp;
elseif ftype == 2   then // high
    WN=WPA*new_wp;
elseif ftype == 3    // stop
    WN=(WPA(1)-WPA(2))*new_wp/2 + ...
        sqrt( (WPA(2)-WPA(1))^2*new_wp^2/4 + WPA(1)*WPA(2));
    WN(2)=WPA(1)*WPA(2)/WN(1);
elseif ftype == 4   then // pass
    WN=(WPA(1)-WPA(2))/(2*new_wp) + ...
    sqrt( (WPA(2)-WPA(1))^2/(4*new_wp^2) + WPA(1)*WPA(2));
    WN(2)=WPA(1)*WPA(2)/WN(1);
end

// finally, transform frequencies from analog to digital if necessary:
if ~strcmp(opt,'z') then //   digital
    wn = ws;  // bilinear transform
else
    wn = WN(:)';
end
endfunction