//////////////////////////////////////////////////////////////////////////// 
// Signal_Processing_Supplementary_toolbox
//
// Copyright (c) by Y. K. Singh, CDAC, Kolkata.  2012
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [y] = ladrfilt(K,C,x)
// LATTICE-LADDER form realization of IIR filters
// ..............................................
// [yl = ladrfilt(K,C,x)
// y = output sequence
// K = LATTICE (reflection) coefficient array
// C = LADDER coefficient array
//  x = input sequence
//
Nx = length(x); y = zeros(1,Nx);
N = length(C); f = zeros(N,Nx); g = zeros(N,Nx+1);
f(N,:) = x;
for n = 2:Nx+1
for m = N:-1:2
f(m-1,n-1) = f(m,n-1) - K(m-1)*g(m-1,n-1);
g(m,n) = K(m-1)*f(m-1,n-1) + g(m-1,n-1);
end
g(1,n) = f(1,n-1);
end
y = C*g(:,2:Nx+1);
endfunction
