//////////////////////////////////////////////////////////////////////////// 
// Signal_Processing_Supplementary_toolbox
//
// Copyright (c) by Y. K. Singh, CDAC, Kolkata.  2012
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function  [P, c] = welch(x, N, M, wintype, nplot)
//WELCH   Power Spectrum Estimation by Welch's method
//-----
//   Usage:      [P,c] = welch(x,N,M,wintype,nplot)
//
//       P : power spectrum by Welch's method
//       c : correlation function = inverse of Welch power spectrum
//       x : input signal vector
//       N : FFT length
//       M : window length
//   wintype = 're' for rectangular
//             'tr' for triagular
//             'hn' for hanning
//             'hm' for hamming (default)
//     nplot = omitting this argument suppresses plotting

//---------------------------------------------------------------
// copyright 1994, by C.S. Burrus, J.H. McClellan, A.V. Oppenheim,
// T.W. Parks, R.W. Schafer, & H.W. Schussler.  For use with the book
// "Computer-Based Exercises for Signal Processing Using MATLAB"
// (Prentice-Hall, 1994).
//---------------------------------------------------------------
[nargout,nargin]=argn(0);
x = x(:);
    w = window(wintype,M);
K = fix((length(x)-M/2)/(M/2));
KMU = K*sum(w.^2);
P = zeros(N,1);
for i=1:K
    st = (M/2)*(i-1) + 1;
    xh=x(st:(st+M-1));
    X = mtlb_fft(xh.*w', N);
    P = P + abs(X).^2;
end
P = P/KMU;
c = mtlb_ifft(P);
c = real(c(1:M/2));
P = P(1:N/2+1);
if( nargin==5 )
  plot((0:N/2)/(N), -28+10*log10(P));
  title('Power Spectral Density Estimate via Welch');
  xlabel('FREQUENCY  (kHz)');
  ylabel('Power/Frequency(dB/Hz)');
end
endfunction
